#include <stdlib.h>
#include <stdio.h>
#include "rlcif.h"	       /* include the rlc include file */

#ifdef HAVE_UI
char tcl_script[] = "proc have_pkts {n} {}";	/* custom UI script */
#endif

static int
rx_packet (int ides, int lay, char flags, char *pkt, int size);

int have_gui;				/* have we got a user interface ? */

int rlc_inst;

int
main (int argc, char *argv[])
{
  unsigned long addr;			/* base address (host byte order) */
  short port;				/* base layer port # */
  char *ev_f = NULL , *po_f = NULL ;	/* possibly log file names */
  addr_list_t addrl;			/* address list ... */
  char *geometry = NULL;


    have_gui = 0;
#ifdef HAVE_UI
    have_gui = 1;
#endif

    /*
    ** now initialize parameters
    ** (addr, port, have_gui, ev_f ?, po_f ? ... )
    */
    addr = 0xe0111213;
    port = 9000;

    if (argc > 1 && !strcmp(argv[1], "-g")) {
	geometry = malloc (strlen(argv[2])+1);
	strcpy (geometry, argv[2]);
    }

    /*
    ** use consecutive addresses, so
    ** provide only the base address.
    */
    addrl.addrs = &addr;	
    addrl.nadd = 1;
    addrl.port = port;

    /*
    ** rlc receiver initialization  ...
    ** and possibly library initialization
    */
    if ( (rlc_inst = rlc_new (1)) < 0 ||
	rlc_set_addr (rlc_inst, &addrl) < 0 ||
	rlc_attach (rlc_inst, 0, (void *)&rx_packet) < 0) {
	fprintf (stderr, "had error in rlc initialization\n");
	exit (1);
    }

    /*
    ** possibly we have logging
    */
    if (ev_f || po_f)
	(void)rlc_rx_log(rlc_inst, ev_f, po_f);

#ifdef HAVE_UI
    if ( have_gui ) {
	if (geometry)
	    rlc_gui_geometry(rlc_inst, geometry);

	if (rlc_gui_enable(rlc_inst) < 0)
	    have_gui = 0;
	/*
	** customize user interface
	*/
	if (rlc_gui_loadtcl(rlc_inst, tcl_script) < 0)
	    have_gui = 0;
    }
#endif

    /*
    ** start receiving ....
    */
    rlc_start(rlc_inst);

    /*
    ** makes thing go ...
    */
    while (EV_TERMINATE != rlc_getevent(1));

    exit(0);
}


/*
** reveived paket handling
*/
static int
rx_packet (int ides, int lay, char flags,
	char *pkt, int size)
{
  static int received;

    received ++;

    free (pkt);

#ifdef HAVE_UI
    /*
    ** display reception progress ?
    */
    if (have_gui)
	rlc_gui_tcl_exec (rlc_inst, NULL, "have_pkts %d", received);
#endif

    /*
    ** use the packet received....
    */
#if 0
    fprintf(stderr, "---rec-sampe: have pkt from lay %d, size %d\n",
	lay, size);
#endif


    /*
    ** if we have done, delete the protocol instance
    */
    if (0) {
	rlc_abort (rlc_inst);
    }

    return(0);
}
